@@ -4,7 +4,7 @@
 
     Support for NumPy and Google style docstrings.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -168,11 +168,10 @@ class Config:
         **If False**::
 
             .. attribute:: attr1
+               :type: int
 
                Description of `attr1`
 
-               :type: int
-
     napoleon_use_param : :obj:`bool` (Defaults to True)
         True to use a ``:param:`` role for each function parameter. False to
         use a single ``:parameters:`` role for all the parameters.
@@ -265,7 +264,7 @@ class Config:
         'napoleon_custom_sections': (None, 'env')
     }
 
-    def __init__(self, **settings) -> None:
+    def __init__(self, **settings: Any) -> None:
         for name, (default, rebuild) in self._config_values.items():
             setattr(self, name, default)
         for name, value in settings.items():
