@@ -574,7 +574,7 @@ def index_role(typ: str, rawtext: str, text: str, lineno: int, inliner: Inliner,
 
 class Index(ReferenceRole):
     def run(self) -> Tuple[List[Node], List[system_message]]:
-        warnings.warn('Index role is deprecated.', RemovedInSphinx40Warning)
+        warnings.warn('Index role is deprecated.', RemovedInSphinx40Warning, stacklevel=2)
         target_id = 'index-%s' % self.env.new_serialno('index')
         if self.has_explicit_title:
             # if an explicit target is given, process it as a full entry
